@@ -1,6 +1,6 @@
 # 2016. 1. 10 by Hans Roh hansroh@gmail.com
 
-__version__ = "0.7.10.2"
+__version__ = "0.7.11.1"
 version_info = tuple (map (lambda x: not x.isdigit () and x or int (x),  __version__.split (".")))
 
 import os, sys
@@ -87,6 +87,7 @@ def configure (
 	use_pool = True,
 	tracking = False,
 	backend = False,
+	dns = []
 ):
 	global _logger, _cb_gateway, _concurrent, _initialized, _timeout
 	global _workers, _que, _allow_redirects, _force_h1
@@ -131,8 +132,8 @@ def configure (
 	
 	socketpool.create (_logger, backend = backend, use_pool = use_pool)
 	dbpool.create (_logger, backend = backend)
-	adns.init (_logger)
-	lifetime.init (_timeout / 2., logger) # maintern interval
+	adns.init (_logger, dns)
+	lifetime.init (_timeout / 2., logger) # maintern interval	
 	if tracking:
 		lifetime.enable_memory_track ()
 	_initialized = True
@@ -306,7 +307,7 @@ def concurrent ():
 	return _concurrent
 		
 def fetchall ():
-	global _workers, _logger, _que, _timeout, _max_conns, _bytesrecv, _concurrent, _finished_total, _max_conns, _force_h1	
+	global _workers, _logger, _que, _timeout, _max_conns, _bytesrecv, _concurrent, _finished_total, _max_conns, _force_h1, _request_total, _bytesrecv
 	
 	if not qsize ():
 		_logger.log ('no item in queue.')
@@ -323,14 +324,13 @@ def fetchall ():
 	#_logger ("creating connection pool", "info")
 	target_socks = min (_workers, qsize ())
 	for i in range (target_socks):
-		_req ()
-	lifetime.poll_dns (True)
-		
+		_req ()	
+	
 	if not _force_h1 and http2.MAX_HTTP2_CONCURRENT_STREAMS > 1:
 		# wait all availabale	
-		while qsize ():
+		while qsize ():			
 			lifetime.lifetime_loop (os.name == "nt" and 1.0 or _timeout / 2.0, 1)
-			if sum ([1 for conn in asyncore.socket_map.values () if conn.get_proto () in H2_PROTOCOLS and conn.connected and not conn.isactive ()]) == _workers:
+			if sum ([1 for conn in asyncore.socket_map.values () if not isinstance (conn, [asyndns.UDPClient, asyndns.TCPClient]) and conn.get_proto () in H2_PROTOCOLS and conn.connected and not conn.isactive ()]) == _workers:
 				#_logger ('%d connection(s) created' % target_socks, 'info')
 				break
 			
@@ -341,14 +341,13 @@ def fetchall ():
 		_max_conns = max (_max_conns, mapsize ())	
 		#print ('--', len (_currents), mapsize (), qsize ())
 		if not mapsize ():
-			break	
-		lifetime.poll_dns (True)
+			break		
 		lifetime.lifetime_loop (os.name == "nt" and 1.0 or _timeout / 2.0, 1)
 	
 	#for each in _currents:
 	#	print ('-- unfinished', each)
 	
-	lifetime._polling = 0	
+	lifetime._polling = 0
 	_duration = timeit.default_timer () - _fetch_started
 	socketpool.cleanup ()
 	dbpool.cleanup ()
@@ -361,7 +360,13 @@ def fetchall ():
 		_max_conns
 		)
 	)
-
+	
+	# reinit for next session
+	_request_total = 0			
+	_finished_total = 0		
+	_max_conns = 0
+	_bytesrecv = 0
+	
 def suspend (timeout):
 	a, b = math.modf (timeout)
 	for i in range (int (b)):		
@@ -391,7 +396,8 @@ def _add (method, url, params = None, auth = None, headers = {}, callback = None
 			_dns_query_req [host] = None
 			_dns_reqs += 1
 			adns.query (host, "A", callback = lambda x: None)		
-		lifetime.poll_dns (True)
+		asyndns.pop_all ()
+		asyncore.loop (0.1, count = 2)
 	
 	#print ('~~~~~~~~~~~~~~~', asyndns.pool.connections)
 	
