@@ -93,7 +93,7 @@ class DNSCache:
 			return callback ([{"name": host, "data": host, "typename": qtype}])
 		
 		try:			
-			asyndns.Request (host, qtype = qtype, protocol = self.prefer_protocol, callback = [self.set, callback])			
+			asyndns.query (host, qtype = qtype, protocol = self.prefer_protocol, callback = [self.set, callback])			
 		except:
 			self.logger.trace (host)
 			hit = [{"name": host, "data": None, "typename": qtype, 'ttl': 60}]
